/* File:      chain.h
   Date:      6/22/02 (written fall 2001)
   Author:    C. Paciorek
   Purpose:   header file for chain.C
   Compile:   
   Usage:     
   Arguments: 
   Requires:  
   Reference: 
*/

class spatial_parm;
class nonspatial_parm;
class nonspatial_vecparm;
class dataset;
class locn_info;
class sparse_cov_mat_nonstat_basis;  /* for accessing kernels */

extern dataset* data;  /* data is global because chain and parms need access */

extern int prior;

extern locn_info* locations;

extern int* idum;                              /* seed for random number generation during proposals */

class chain{

/* This class initializes and holds the parameters, tells each parameter to do a step and reports results */
   
public:
  chain(int num_gridpoints_in,string dat_fn_in,string years_fn_in,string init_fn_in,string final_fn_in,string parm_fn_in,string out_fn_in,int num_its_in,int storage_interval_in,int storage_space_in,int fastparm_interval_in,string datacov_fn_in);  /* initialization */
  ~chain(void);
  void store(int it_cur);  /* stores results of each iteration */
  void do_a_step(int it_cur);  /* does a round/iteration of MCMC, updating each parameter */
  void report(int it_cur,int status);  /* reports final results */
private:
  int num_gridpoints;
  double logpost;
  string out_fn; /* name of file to send output to */
  string final_fn;/* name of file to send final parm values to */
  string idum_fn; /* name of file to store random seed in */
  mat_pp<double> *storage0;  /* storage for log posterior */
  mat_pp<double> *storage1;  /* storage for parameter iterates */
  mat_pp<double> *storage2;  /* storage for parameter iterates */
  mat_pp<double> *storage3;  /* storage for parameter iterates */
  mat_pp<double> *storage4;  /* storage for parameter iterates */
  mat_pp<double> *storage5;  /* storage for parameter iterates */
  mat_pp<double> *storage6;  /* storage for parameter iterates */
  mat_pp<double> *storage7;  /* storage for parameter iterates */
  int num_its;
  int storage_interval; /* store every storage_interval iterates */
  int storage_space;   /* number of stored iterates in each output file */
  int fastparm_interval; /* sample the fast-mixing parms every this many samples */
  int it_run;    /* iteration in current block of its */
  int block;
  /* parameters in model  */
  spatial_parm* alpha;
  spatial_parm* beta;
  spatial_parm* eta;

  /* data covariance parameters */
  nonspatial_parm* delta;
  nonspatial_parm* nu;
  nonspatial_parm* data_kappa;

  /* basis kernel non-stationary approach */
  nonspatial_vecparm* eigenval1vec;
  nonspatial_vecparm* eigenval2vec;
  nonspatial_vecparm* eigenvec1vec;

  /* for full kernel approach
     spatial_parm* eigenval1; 
     spatial_parm* eigenval2; 
     spatial_parm* eigenvec1; 
     spatial_parm* eigenvec2; 
     */
  

/* initial values and ongoing storage space for current parameters */
  double* alpha_init;
  double* beta_init;
  double* eta_init;

  /* for full kernel approach */
  double* eigenval1_init;
  double* eigenval2_init;
  double* eigenvec1_init;
  double* eigenvec2_init;

  int* kernlocs;
  ofstream output;
};





