/* File:      chebevD.C
   Date:      September 2002  
   Author:    Numerical recipes in C, transcribed by C. Paciorek
   Purpose:   support code for bessikD.C
   Compile:   g++ -c
   Usage:     
   Arguments: 
   Requires:  base.h
   Reference: Numerical Recipes in C
*/
#include "base.h"

double chebevD(double a, double b, double c[], int m, double x) /* Chebyshev evaluation: All arguments are input. c[0..m-1] is an array of Chebyshev coefficients, the first m elements of c output from chebft (which must have been called with the same a and b). The Chebyshev polynomial P m-1 k=0 ck Tk (y)   c0=2 is evaluated at a point y = [x   (b+a)=2]=[(b   a)=2], and the result is returned as the function value. */
 { 
   double d=0.0,dd=0.0,sv,y,y2; 
   int j; 
   if ((x-a)*(x-b) > 0.0) cerr << "x not in range in routine chebev\n";
   y2=2.0*(y=(2.0*x-a-b)/(b-a)); // Change of variable. 
   for (j=m-1;j>=1;j--) { // Clenshaw's recurrence. 
     sv=d; d=y2*d-dd+c[j]; dd=sv; 
   } 
   return y*d-dd+0.5*c[0]; // Last step is different. 
 }
