/* File:      cov_mat.h
   Date:      7/9/02  (code written fall 2001)
   Author:    C. Paciorek
   Purpose:   header file for cov_mat.C
   Compile:   
   Usage:     
   Arguments: 
   Requires:  
   Reference: 
*/

const double TOL=.000000000001;
const double smoothness=4.0;

class locn_info;
extern locn_info* locations;

extern "C" void smbfct_(int* n,int* xadj,int* adjncy,int* perm,int* invp,int* xlnz,int* maxlnz,int* xnzsub,int* nzsub,int* maxsub,int* rchlnk,int* mrglnk,int* marker); /* Fortran routine for symb. factorization */

extern "C" void gsfct_(int* n,int* xlnz,double* lnz,int* xnzsub,int* nzsub,double *diag,int* link,int* first,double *tmp,double* eps,int* war);  /* Fortran routine for numeric factorization */

class sparse_cov_mat{
public:
  sparse_cov_mat(int size_in);
  ~sparse_cov_mat(void);
  void sym_chol(void);                /* symbolic factorization -- sets up sparse structure */
  void gen_chol(int warn);                /* numeric factorization */
  void solve(double* rhs_in, double* output);  /* calcs C^{-1}b */
  void Lmult(double* b, double* out);       /* multiplies L times vector b */
  void Ltmult(double* b, double* out);       /* multiplies L^T times vector b */
  void Lcheck(double* newvec,double* oldvec);
  void Linvmult(double* rhs_in, double* output);       /* multiplies L times vector b */
  double det_log_squ_root(void);        /* calc sqrt of log(det) via sum of log od Cholesky diagonals */
  double* lnz_C;            /* public so alpha and beta spat_parm have access */
  double* diag_C;           /* holds diag values */
protected:   
  /* protected because other cov_mat types need access */
  /* _C indicates corr matrix, _L is Cholesky factor */
  int size;
  int num_dim;
  double* lnz_C_stored; 
  double* diag_C_stored;           /* holds diag values */
  double *diag_L;
  double *lnz_L;            /* holds nonzero off-diag values by column, then by row */
  double* lnz_L_stored;     /* stores values for restoration if proposal not accepted */
  double* diag_L_stored;
  double* distances;

  int maxlnz;               /* length of lnz */
  int maxsub;               /* length of nzsub */
  int *nzsub;               /* row indices of the off-diags, with repeats eliminated */
  int *xlnz;                /* vector of pointers (1,...,n+1) of where the elements of a column start -- xlnz[0] is where the elements of col 1 start; xlnz[1] is where col 2 starts,  */
  int *xnzsub;              /* vector of pointers (1,...,n+1) of start of column in nzsub -- xnzsub[0] is where the row indices of column 1 start; xnzsub[1] for column 2, etc. */
  /* adjancency and permutation information are part of locn_info class -- cov_mat is a friend of locations */
private:
};

/* stat is only used for parameters, not for data covariance (see fullstat) */
class sparse_cov_mat_stat : public sparse_cov_mat{
public:
  sparse_cov_mat_stat(int size_in,double* corr_parm_in);
  ~sparse_cov_mat_stat(void);  
  void update(int accepted);       /* update current and stored values depending on whether proposed values are accepted */
  void fill(int first);            /* calc values in matrix */
  double fillij(int index);  /* calc ijth element of matrix */
  void sym_chol(void);
  void gen_chol_Csum(void);                /* numeric factorization */
  int solve_Csum(double* rhs_in, double* output);
  void fill_Csum(int which);
  void Cparmmult(double* b, double* out); /* multiplies by vector b */
  void Cdatamult(double* b, double* out); /* multiplies by vector b */
  void Cdata2mult(double* b, double* out); /* multiplies by vector b */
  void add_info(double* data_diag_C_in,double* data_lnz_C_in,double* sigma_in,double divisor_in,double* etahat,double* delta);
private:
  double* corr_parm;

  double* lnz_Csum;
  double* diag_Csum;
  double* lnz_L_Csum;
  double* diag_L_Csum;
  
  double* data_diag_C;
  double* data_lnz_C;
  double divisor; /* 51 for alpha and 11050 for beta */
  double* sigma;  /* variance of spatial parm */
  double* etahat; /* MLE for variances used in approx cov of eta */
  double* delta;  /* used in constructing cov(log(etahat)) */
};

class sparse_cov_mat_fixed : public sparse_cov_mat{
public:
  sparse_cov_mat_fixed(int size_in,double* variances_in, double* noise_var_in,string file_in);
  ~sparse_cov_mat_fixed(void);  
  void update(int accepted);        /* update current and stored values depending on whether proposed values are accepted */
  void fill(int which,int first);   /* calc values in matrix */
  double fillij(int isub,int col);  /* calc ijth element of matrix */
  void sym_chol(void);
  double* noise_var;
  double* lnz_R;
  double* diag_R;
private:
  double* variances;
  mat_pp<double>* corrmat_in;
};

class sparse_cov_mat_fullstat : public sparse_cov_mat{
public:
  sparse_cov_mat_fullstat(int size_in,double* variances_in, double* noise_var_in,double* nu_in,double* corr_parm_in);
  ~sparse_cov_mat_fullstat(void);  
  void update(int accepted);       /* update current and stored values depending on whether proposed values are accepted */
  void fill(int which,int first);            /* calc values in matrix */
  double fillij(int index);  /* calc ijth element of matrix */
  void sym_chol(void);
/*  void gen_chol2(int warn); */
 /* double det_log_squ_root(void); */
  double* noise_var;
  double* nu;
  double* corr_parm;    
  double* lnz_R;
  double* diag_R;
private:
  double* variances;
  double* lnz_R_stored;
  double* diag_R_stored;
};

class sparse_cov_mat_nonstat_basis : public sparse_cov_mat{
public:
  sparse_cov_mat_nonstat_basis(int size_in,double* variances_in, double* noise_var_in,double* nu_in,double* kappa1_in,double* kappa2_in,double** kern_eigenval1_in,double** kern_eigenval2_in,double** kern_eigenvec1_in);
  ~sparse_cov_mat_nonstat_basis(void);
  void update(int accepted);  /* update current and stored values depending on whether proposed values are accepted */
  void sym_chol(void);     
  void fill(int which, int first);       /* calc values in matrix */
  double fillij(int i, int j,double u,double v);  /* calc ijth element of matrix */
/*  void gen_chol2(int warn); */
 /* double det_log_squ_root(void); */
  double* noise_var;
  double* nu;
  double* kappa1;
  mat_pp<double>* kernels;
  double* lnz_R;
  double* diag_R;
private:
  /* _R indicates corr matrix, _C indicates associated cov matrix; _L is Cholesky factor 
   _stored stores values for restoration if proposal is not accepted */

  double** kern_eigenval1;
  double** kern_eigenvec1;
  double** kern_eigenval2;
  double* variances;

  mat_pp<double>* basis_kernels;

  double* vec;
  double* summat;

  double* lnz_R_stored;
  double* diag_R_stored;

  double* Euc_u;                /* Euclidean u distance between pair of points for each element of cov matrix */
  double* Euc_v;                /* Euclidean v distance between pair of points for each element of cov matrix */
};

class sparse_cov_mat_nonstat : public sparse_cov_mat{
public:
  sparse_cov_mat_nonstat(int size_in,double* variances_in, double* noise_var_in,double* eigenval1_in,double* eigenval1_in,double* eigenvec1_in,double* eigenvec2_in);
  ~sparse_cov_mat_nonstat(void);
  void update(int accepted);  /* update current and stored values depending on whether proposed values are accepted */
  void sym_chol(void);     
  void fill(int which, int first);       /* calc values in matrix */
  double fillij(int i, int j,double u,double v);  /* calc ijth element of matrix */
  double* noise_var;
  double* lnz_R;
  double* diag_R;
private:
  /* _R indicates corr matrix, _C indicates associated cov matrix; _L is Cholesky factor 
   _stored stores values for restoration if proposal is not accepted */
  mat_pp<double>* eigenvecs;

  double* eigenval1;
  double* eigenvec1;
  double* eigenval2;
  double* eigenvec2;
  double* variances;

  mat_pp<double>* kernel;
  mat_pp<double>* eigenvecs_stored;

  double* eigenval_mat;
  double* vec;
  double* chol;
  double* summat;
  double* tmp;

  double* lnz_R_stored;
  double* diag_R_stored;

  double* Euc_u;                /* Euclidean u distance between pair of points for each element of cov matrix */
  double* Euc_v;                /* Euclidean v distance between pair of points for each element of cov matrix */
};










