/* File:      dataset.C
   Date:      6/22/02 (code written fall 2001)
   Author:    C. Paciorek
   Purpose:   code for storing data, calculating sufficient statistics, calculating likelihood
   Compile:   g++ -c 
   Usage:     
   Arguments: 
   Requires:  dataset.h, base.h, permtools.h, cov_mat.h, locations.h (for current permutation), generic_functions.h
   Reference: 
*/

#include "base.h"
#include "dataset.h"
#include "permtools.h"           // permrv
#include "cov_mat.h"
#include "locations.h"           // permutation

void dumb3(void){
  int i=7;
}

dataset::dataset(string in_fn,string years_in_fn,int num_years_in, int num_gridpoints_in,double* ints_in, double* slopes_in,double* variances_in,double* delta_in,double* nu_in, double* kappa1_in,double* kappa2_in,double** eigenval1_in,double** eigenval2_in, double** eigenvec1_in, double** eigenvec2_in,string datacorr_fn_in ){   // kernel parms are passed through to covariance matrix
  int i,year;
  int accepted=ACCEPT;                          // first time through, acceptance makes proposed and stored values the same
  num_gridpoints=num_gridpoints_in;

  tmpvec1=new double[num_gridpoints];
  tmpvec2=new double[num_gridpoints];
  tmpvec3=new double[num_gridpoints];

  etahat=new double[num_gridpoints];
  // test-train split by years
  num_years=num_years_in;
  years = new int[num_years];

  mat_pp<int> tmpyears(1,num_years);
  tmpyears.read(years_in_fn,BY_ROW,RAGGED);    // reads in the years used
  tmpyears.getvec(0,ROW,&years);           
 
  num_train_years=0;
  for(year=0;year<num_years;year++){
    if(years[year]){
      years[year]=1;
      num_train_years++;
    }
  }

  num_test_years=num_years-num_train_years;
  if(num_test_years){
    logpost_test=new double[num_test_years];
    SSE_test=new double[num_test_years];
    SSSE_test=new double[num_test_years];
  }

  // parameters
  ints=ints_in;                                
  slopes=slopes_in;
  variances=variances_in;

  // data values and data statistics
  values=new mat_pp<double>(num_years,num_gridpoints);  // data stored as each row a location
  values->read(in_fn,BY_COL,RAGGED);         // input data files have each row a year
  sum_data=new double[num_gridpoints];            // sum_{t} Y_{l,t}, t=-25,...,25
  sum_data_by_t=new double[num_gridpoints];       // sum_{t} t*Y_{l,t}, t=-25,...,25 
  for(i=0;i<num_gridpoints;i++){
    sum_data[i]=0.0;
    sum_data_by_t[i]=0.0;
  }
  sum_t_squ=0;                                  // sum_{t} t*t, t=-25,...,25 
  for(year=0;year<num_years;year++){
    permrv(&num_gridpoints,values->vals[year],locations->perm,PERMSTART1);    // need to permute data to match parameters
    if(years[year]){
      for(i=0;i<num_gridpoints;i++){
	sum_data[i]+=values->vals[year][i];
	sum_data_by_t[i]+=values->vals[year][i]*(year-25);
      }
      sum_t_squ+=(year-25)*(year-25);
    }
  }

  for(year=0;year<num_years;year++){
    if(years[year]){
      for(i=0;i<num_gridpoints;i++){ // Y*(x)=Y(x)-a(x)-b(x)t; var=sum(Y*(x)^2)/T
	etahat[i]+=pow((double)values->vals[year][i]-sum_data[i]/num_train_years-(double)sum_data_by_t[i]/sum_t_squ*(double)(year-25.0),2.0);
      }
    }
  }
  for(i=0;i<num_gridpoints;i++){
    etahat[i]=etahat[i]/(double)num_train_years;
  }

  delta=delta_in;

  // cov matrix stuff
  if(prior==STATIONARY){
    stat_cov=new sparse_cov_mat_fullstat(num_gridpoints,variances_in,delta_in,nu_in,kappa1_in);  // data covariance matrix - nonstationary
    nonstat_cov=0;
    fixed_cov=0;
    datacov_change(DATA_KAPPA,FIRST);
    evaluate_loglik_diff(DATA_KAPPA,FIRST);                      // calc initial likelihood
    update(accepted,DATA_KAPPA);                                  // update stored likelihood
  }
  if(prior==NON_STATIONARY){
    // basis kernel covariance
    nonstat_cov=new sparse_cov_mat_nonstat_basis(num_gridpoints,variances_in,delta_in,nu_in,kappa1_in,0,eigenval1_in,eigenval2_in,eigenvec1_in);  // data covariance matrix - nonstationary
    /* full kernel approach 
       // if I use the following, I need to pass eigen stuff in as double**
    nonstat_cov=new sparse_cov_mat_nonstat(num_gridpoints,variances_in,delta_in,*eigenval1_in,*eigenval2_in,*eigenvec1_in,*eigenvec2_in);  // data covariance matrix - nonstationary
    */
    stat_cov=0;
    fixed_cov=0;
    datacov_change(DATA_COVAR_EIGVEC,FIRST);
    evaluate_loglik_diff(DATA_COVAR_EIGVEC,FIRST);                      // calc initial likelihood
    update(accepted,DATA_COVAR_EIGVEC);                                  // update stored likelihood
  }
  if(prior==FIXED){
    fixed_cov=new sparse_cov_mat_fixed(num_gridpoints,variances_in,delta_in,datacorr_fn_in);  // data covariance matrix - nonstationary
    stat_cov=0;
    nonstat_cov=0;
    datacov_change(DELTA,FIRST);
    evaluate_loglik_diff(DELTA,FIRST);                      // calc initial likelihood
    update(accepted,DELTA);                                  // update stored likelihood
  }
}
 


dataset::~dataset(void){
  delete values;
  delete [] years;
  if(num_test_years){
    delete [] logpost_test;
    delete [] SSE_test;
    delete [] SSSE_test;
 }
  delete [] sum_data;
  delete [] sum_data_by_t;
  if(prior==STATIONARY){
    delete stat_cov;
  }
  if(prior==NON_STATIONARY){
    delete nonstat_cov;
  }
  if(prior==FIXED){
    delete fixed_cov;
  }
  delete [] tmpvec1;
  delete [] tmpvec2;
  delete [] tmpvec3;
}

double* dataset::get_diag_C(void){
  if(prior==STATIONARY){
    return(stat_cov->diag_C);
  }
  if(prior==NON_STATIONARY){
    return(nonstat_cov->diag_C);
  }
  if(prior==FIXED){
    return(fixed_cov->diag_C);
  }
  return(0);
}

double* dataset::get_lnz_C(void){
  if(prior==STATIONARY){
    return(stat_cov->lnz_C);
  }
  if(prior==NON_STATIONARY){
    return(nonstat_cov->lnz_C);
  }
  if(prior==FIXED){
    return(fixed_cov->lnz_C);
  }
  return(0);
}

double* dataset::get_diag_R(void){
  if(prior==STATIONARY){
    return(stat_cov->diag_R);
  }
  if(prior==NON_STATIONARY){
    return(nonstat_cov->diag_R);
  }
  if(prior==FIXED){
    return(fixed_cov->diag_R);
  }
  return(0);
}

double* dataset::get_lnz_R(void){
  if(prior==STATIONARY){
    return(stat_cov->lnz_R);
  }
  if(prior==NON_STATIONARY){
    return(nonstat_cov->lnz_R);
  }
  if(prior==FIXED){
    return(fixed_cov->lnz_R);
  }
  return(0);
}


void dataset::update(int accepted,int which){
  // if accepted, stored value becomes current proposed values, if not, restore the stored value as the current value;  depending on parameter being proposed, some portions of likelihood stay fixed
  if(accepted){
    loglik=loglik_alpha+loglik_beta+loglik_gamma;
    if(loglik>100000){
      dumb3();
    }
  }
  if(which==ALPHA){
    if(accepted){
      loglik_alpha_stored=loglik_alpha;
      loglik_gamma_stored=loglik_gamma;
    }
    else{
      loglik_alpha=loglik_alpha_stored;
      loglik_gamma=loglik_gamma_stored;
    }
  }
  if(which==BETA){
    if(accepted){
      loglik_beta_stored=loglik_beta;
      loglik_gamma_stored=loglik_gamma;
    }
    else{
      loglik_beta=loglik_beta_stored;
      loglik_gamma=loglik_gamma_stored;
    }
  }
  if(which==ETA || which==DELTA || which==NU || (prior==STATIONARY && which==DATA_KAPPA) || (prior==NON_STATIONARY && (which==DATA_EIG_KAPPA || which==DATA_COVAR_EIGVAL || which==DATA_COVAR_EIGVEC))){
    if(prior==STATIONARY){
      stat_cov->update(accepted);
    }
    if(prior==NON_STATIONARY){
      nonstat_cov->update(accepted);
    }
    if(prior==FIXED){
      fixed_cov->update(accepted);
    }
    if(accepted){
      loglik_gamma_stored=loglik_gamma;
      loglik_alpha_stored=loglik_alpha;
      loglik_beta_stored=loglik_beta;
    }
    else{
      loglik_gamma=loglik_gamma_stored;
      loglik_alpha=loglik_alpha_stored;
      loglik_beta=loglik_beta_stored; 
    }
  }
}

void dataset::datacov_change(int which,int first){
  if(prior==STATIONARY){
      stat_cov->fill(which,first);                         // find new data cov matrix
      stat_cov->gen_chol(WARN);                                // calc its Cholesky
      if(first==FIRST){            // seems redundant with dataset::update 
	stat_cov->update(ACCEPT);
      }
  }
  if(prior==NON_STATIONARY){
    nonstat_cov->fill(which,first);                         // find new data cov matrix
    nonstat_cov->gen_chol(WARN);                                // calc its Cholesky
    if(first==FIRST){           // seems redundant with dataset::update 
      nonstat_cov->update(ACCEPT);
    }
  }
  if(prior==FIXED){
    fixed_cov->fill(which,first);                         // find new data cov matrix
    fixed_cov->gen_chol(WARN);                                // calc its Cholesky
    if(first==FIRST){           // seems redundant with dataset::update 
      fixed_cov->update(ACCEPT);
    }
  }
}


double dataset::evaluate_loglik_diff(int which,int first){
  // calculates loglikelihood, but only portion that changes with the parameter being proposed
  int year;
  double tmp;
  
  if(first==FIRST){
    loglik_alpha=0.0; loglik_alpha_stored=0.0;
    loglik_beta=0.0; loglik_beta_stored=0.0;
  }

  if(prior==STATIONARY){
    if(which==ALPHA){                    // data cov matrix stays fixed
      stat_cov->solve(ints,tmpvec1);   // lik involves inverse of data cov matrix mult by intercepts vector
      loglik_alpha=-0.5*(-2*dot_prod(num_gridpoints,sum_data,tmpvec1)+num_train_years*dot_prod(num_gridpoints,ints,tmpvec1));
      return(loglik_alpha-loglik_alpha_stored);
    }
    if(which==BETA){                     // data cov matrix stays fixed
      stat_cov->solve(slopes,tmpvec2); // lik involves inverse of data cov matrix mult by slopes vector
      loglik_beta=-0.5*(-2*dot_prod(num_gridpoints,sum_data_by_t,tmpvec2)+sum_t_squ*dot_prod(num_gridpoints,slopes,tmpvec2));
      return(loglik_beta-loglik_beta_stored);
    }
    if(which==DATA_KAPPA || which==ETA || which==DELTA || which==NU){
    

      stat_cov->solve(ints,tmpvec1);
      stat_cov->solve(slopes,tmpvec2);

      loglik_alpha=-0.5*(-2*dot_prod(num_gridpoints,sum_data,tmpvec1)+num_train_years*dot_prod(num_gridpoints,ints,tmpvec1));
      
      loglik_beta=-0.5*(-2*dot_prod(num_gridpoints,sum_data_by_t,tmpvec2)+sum_t_squ*dot_prod(num_gridpoints,slopes,tmpvec2));
      tmp=0.0;

      for(year=0;year<num_years;year++)                         // sum_{t} Y_{l,t} C^{-1} Y_{l,t}
	{
	  if(years[year]){
            /*
              // this is version with t likelihood
              for(int i=0;i<num_gridpoints;i++){
              tmpvec1[i]=values->vals[year][i]-ints[i]-slopes[i]*(double)(year-25.0);
              }
              stat_cov->solve(tmpvec1,tmpvec3);
              tmp+=0.5*(1.0+num_gridpoints)*log(1.0+dot_prod(num_gridpoints,tmpvec1,tmpvec3));
            */
            stat_cov->solve(values->vals[year],tmpvec3);
            tmp+=dot_prod(num_gridpoints,values->vals[year],tmpvec3);
 
	  }
	}
      loglik_gamma=-num_train_years*stat_cov->det_log_squ_root() - 0.5*tmp;
      // 1*tmp for t

      return((loglik_gamma+loglik_alpha+loglik_beta)-(loglik_gamma_stored+loglik_alpha_stored+loglik_beta_stored));
    }
    return(0.0);
  }
  if(prior==NON_STATIONARY){
    if(which==ALPHA){                    // data cov matrix stays fixed
      nonstat_cov->solve(ints,tmpvec1);   // lik involves inverse of data cov matrix mult by intercepts vector
      loglik_alpha=-0.5*(-2*dot_prod(num_gridpoints,sum_data,tmpvec1)+num_train_years*dot_prod(num_gridpoints,ints,tmpvec1));
      return(loglik_alpha-loglik_alpha_stored);
    }
    if(which==BETA){                     // data cov matrix stays fixed
      nonstat_cov->solve(slopes,tmpvec2); // lik involves inverse of data cov matrix mult by slopes vector
      loglik_beta=-0.5*(-2*dot_prod(num_gridpoints,sum_data_by_t,tmpvec2)+sum_t_squ*dot_prod(num_gridpoints,slopes,tmpvec2));
      
      return(loglik_beta-loglik_beta_stored);
    }
    if(which==ETA || which==DELTA || which==NU || which==DATA_EIG_KAPPA || which==DATA_COVAR_EIGVAL || which==DATA_COVAR_EIGVEC){  // these parms affect data cov matrix

      nonstat_cov->solve(ints,tmpvec1);
      nonstat_cov->solve(slopes,tmpvec2);

      loglik_alpha=-0.5*(-2*dot_prod(num_gridpoints,sum_data,tmpvec1)+num_train_years*dot_prod(num_gridpoints,ints,tmpvec1));
      
      loglik_beta=-0.5*(-2*dot_prod(num_gridpoints,sum_data_by_t,tmpvec2)+sum_t_squ*dot_prod(num_gridpoints,slopes,tmpvec2));

      tmp=0.0;
      for(year=0;year<num_years;year++)                         // sum_{t} Y_{l,t} C^{-1} Y_{l,t}
	{
	  if(years[year]){
	    nonstat_cov->solve(values->vals[year],tmpvec3);
	    tmp+=dot_prod(num_gridpoints,values->vals[year],tmpvec3);
	  }
	}
      loglik_gamma=-num_train_years*nonstat_cov->det_log_squ_root() - 0.5*tmp;
      return((loglik_gamma+loglik_alpha+loglik_beta)-(loglik_gamma_stored+loglik_alpha_stored+loglik_beta_stored));
    } 
    return(0.0);
  }
  if(prior==FIXED){
    if(which==ALPHA){                    // data cov matrix stays fixed
      fixed_cov->solve(ints,tmpvec1);   // lik involves inverse of data cov matrix mult by intercepts vector
      loglik_alpha=-0.5*(-2*dot_prod(num_gridpoints,sum_data,tmpvec1)+num_train_years*dot_prod(num_gridpoints,ints,tmpvec1));
      return(loglik_alpha-loglik_alpha_stored);
    }
    if(which==BETA){                     // data cov matrix stays fixed
      fixed_cov->solve(slopes,tmpvec2); // lik involves inverse of data cov matrix mult by slopes vector
      loglik_beta=-0.5*(-2*dot_prod(num_gridpoints,sum_data_by_t,tmpvec2)+sum_t_squ*dot_prod(num_gridpoints,slopes,tmpvec2));
      return(loglik_beta-loglik_beta_stored);
    }
    if(which==DATA_KAPPA || which==ETA || which==DELTA){

      fixed_cov->solve(ints,tmpvec1);
      fixed_cov->solve(slopes,tmpvec2);

      loglik_alpha=-0.5*(-2*dot_prod(num_gridpoints,sum_data,tmpvec1)+num_train_years*dot_prod(num_gridpoints,ints,tmpvec1));
      
      loglik_beta=-0.5*(-2*dot_prod(num_gridpoints,sum_data_by_t,tmpvec2)+sum_t_squ*dot_prod(num_gridpoints,slopes,tmpvec2));

      tmp=0.0;
      for(year=0;year<num_years;year++)                         // sum_{t} Y_{l,t} C^{-1} Y_{l,t}
	{
	  if(years[year]){
	    fixed_cov->solve(values->vals[year],tmpvec3);
	    tmp+=dot_prod(num_gridpoints,values->vals[year],tmpvec3);
	  }
	}
      loglik_gamma=-num_train_years*fixed_cov->det_log_squ_root() - 0.5*tmp;
      return((loglik_gamma+loglik_alpha+loglik_beta)-(loglik_gamma_stored+loglik_alpha_stored+loglik_beta_stored));
    }
    return(0.0);
  }
  return(0.0);
}

double dataset::evaluate_logpost(void){
  // calculates logposterior, for test time points
  int year,i,count;
  double sum;
  if(prior==STATIONARY){
    logpost_det=-num_test_years*stat_cov->det_log_squ_root();
  }
  if(prior==NON_STATIONARY){
    logpost_det=-num_test_years*nonstat_cov->det_log_squ_root();
  }
  if(prior==FIXED){
    logpost_det=-num_test_years*fixed_cov->det_log_squ_root();
  }
  sum=logpost_det;
  count=0;
  for(year=0;year<num_years;year++){
    if(!years[year]){
      for(i=0;i<num_gridpoints;i++){
	tmpvec2[i]=values->vals[year][i]-ints[i]-slopes[i]*(year-25.0);
      }
      if(prior==STATIONARY){
	stat_cov->solve(tmpvec2,tmpvec1);
      }
      if(prior==NON_STATIONARY){
	nonstat_cov->solve(tmpvec2,tmpvec1);
      }
      if(prior==FIXED){
	fixed_cov->solve(tmpvec2,tmpvec1);
      }
            logpost_test[count]=-0.5*dot_prod(num_gridpoints,tmpvec2,tmpvec1);
            //logpost_test[count]=-0.5*(1.0+num_gridpoints)*log(1.0+dot_prod(num_gridpoints,tmpvec2,tmpvec1));  // for t lik
      SSE_test[count]=0.0;
      SSSE_test[count]=0.0;
      for(i=0;i<num_gridpoints;i++){
        SSE_test[count]+=pow(values->vals[year][i]-ints[i]-slopes[i]*(year-25.0),2.0);
        SSSE_test[count]+=pow(values->vals[year][i]-ints[i]-slopes[i]*(year-25.0),2.0)/variances[i];
      }
      sum+=logpost_test[count];
      count++;
    }
  }
  return(sum);
}











