/* File:      density.C
   Date:      12/31/01  (written fall 2001)
   Author:    C. Paciorek
   Purpose:   evaluates various log-density functions 
   Compile:   g++ -c  
   Usage:     
   Arguments: 
   Requires:  density.h, base.h
   Reference: 
*/


#include "base.h"
#include "density.h"


/* These functions evaluate the log density of arg and return the value of the kernel of the density without evaluating the constant */

// Lognormal
double lognormal_dens(double arg,double mean, double sd)
{
  if(arg<0 || sd<=0) {return(DENSITY_ERROR);}
  return(-log(arg)-0.5*(log(arg)-mean)*(log(arg)-mean)/(sd*sd));
}

// Gamma, mean=alpha*beta
double gamma_dens(double arg,double alpha,double beta)
{
  if(arg<0 || alpha<=0 || beta <=0) {return(DENSITY_ERROR);}
  return((alpha-1)*log(arg)-arg/beta);
}

// Inverse gamma: mean=beta/(alpha-1)
double invgamma_dens(double arg,double alpha,double beta){
  if(alpha<=0.0 || beta<=0.0){
    return(DENSITY_ERROR);
  }
  // prior is on sigma2 not log(sigma) so Jacobian is included here
  return(-2.0*alpha*arg-beta/(exp(2.0*arg)));
}

// Univariate Normal
double normal_dens(double arg,double mean, double sd)
{
  if(sd <=0) {return(DENSITY_ERROR);}
  return(-0.5*(arg-mean)*(arg-mean)/(sd*sd));
}

// Multivariate Normal
/*
double mvn_dens(int size,LaVectorDouble* values,double sd){
  double sum=0.0; int i;
  if(sd <=0) {return(DENSITY_ERROR);}
  for(i=0;i<size;i++){
    sum+=(*values)(i)*(*values)(i);
  }
  return(-0.5*sum/(sd*sd));
}
*/

// Multivariate Normal
double mvn_dens(int size,double* values,double sd){
  double sum=0.0; int i;
  if(sd <=0) {return(DENSITY_ERROR);}
  for(i=0;i<size;i++){
    sum+=values[i]*values[i];
  }
  return(-0.5*sum/(sd*sd));
}

// Uniform 
double uniform_dens(double arg,double low,double up){
  if(arg<=up && arg>=low){
    return(0.0);
  }
  else{
    return(NEG_INFTY);
  }
}

// not logged
// arg=exp(uniform RV); low and up are already exponentiated
double log_uniform(double arg,double low,double up){
  if(arg<=up && arg>=low){
    return(1.0/((log(up)-log(low))*arg));
  }
  else{
    return(0.0);
  }
}
  




