/* File:      locations.h
   Date:      12/31/01  (written fall 2001)
   Author:    C. Paciorek
   Purpose:   header file for locations.C
   Compile:   
   Usage:     
   Arguments: 
   Requires:  
   Reference: 
*/


const int GRID=0;
const int NON_GRID=1;

const int NO_PERMUTE=0;
const int PERMUTE=1;

extern int printout;

extern "C" void genqmd_(int* n,int* xadj,int *iadj,int* perm,int* invprm,int* deg,int* marker,int* rchset,int* nbrhd,int *qsize,int* qlink,int* nofsub);   /* Fortran function for permuting locations to achieve Cholesky sparseness */

class locn_info {
public:
  /* a neighbor is defined as a location within range of a given location */
  locn_info(string lat_in_fn,string lon_in_fn,string kernel_lat_in_fn,string kernel_lon_in_fn,string perm_fn,string invp_fn,int format_in,double range_in,int num_latpoints_in,int num_lonpoints_in,int num_kernels_in);
  ~locn_info(void);
  void fill_adj(void);                       /* determines which locations are neighbors (within range s.t. may have non-zero correlation, and fills adjacency structures */
  void permute(void);                        /* permutes locations to achieve Cholesky sparseness */
  double ang_distance(int x, int y, int units, int permute);  /* finds distance between locations x and y */
  void euc_distance(int x,int y,double *euclid_u,double *euclid_v,int permute); /* calcs Euclidean u and v distances between locations x and y */
  void calc_locn_kernel_angdists(int permute); /* calculate ang dist in both lat and lon between locations and basis kernels */
  double range;                              /* maximal distance for non-zero correlation; currently in RADIANS */
  int num_gridpoints;                        /* number of locations: 3024 = 21 * 144 now */
  int *xadj;                                 /* for each location, pointer to start of neighbors in adjncy - Fortran style indexing */   
  int *adjncy;                               /* adjacency vector in George & Liu form - Fortran style indexing */
  int* perm;  /* permutation of locations to achieve Cholesky sparseness:  perm = {2,3,4,1,5} means that first location is the orig 2nd one, 2nd is the original 3rd one, etc. */
  int* invp;  /* inverse of perm */  
  int num_kernels;
  mat_pp<double>* locn_kernel_dists;      /* stores pairwise latitude distances of kernels and locations */
private:
  int format;                                /* GRID or NON_GRID -- non-GRID not yet implemented */
  int num_latpoints;                            /* 21 for now: 70-20 N */
  int num_lonpoints;                            /* 144 for now */
  double *lat;                               /* vector of lats for the locations */
  double *lon;                               /* vector of lons for the locations */
  double *lat_grid;                          /* set of latitude grid points */
  double *lon_grid;                          /* set of longitude grid points */

  int **neighbor_ptr;                        /* stores indices of neighbors of each location - C style */
  int *num_neighbors;                        /* number of neighbors for each location */
  int adjncy_len;

  double* kernel_lat;                        /* vector of lats for the kernels */
  double* kernel_lon;                        /* vector of lons for the kernels */
  mat_pp<double>* u;
  mat_pp<double>* v;

};




