/* File:      parm.h
   Date:      6/22/02 (revised from fall 2001 version)
   Author:    C. Paciorek
   Purpose:   header file for parm.C
   Compile:   
   Usage:     
   Arguments: 
   Requires:  
   Reference: 
*/


typedef double(*transform_fxn_ptr) (double,double);    /* used as type when passing transform fxns;  second double is df, only used in chisq transformation */

typedef double(*prior_dens_fxn_ptr) (double,double,double);    /* used as type when passing logdensity functions; first arg is value to evaluate at, 2nd is first parm, 3rd is second parm */ 

class chain;                          /* chain is used as friend */
class dataset;
class sparse_cov_mat_stat;
class hyperparm_vec;
class hyperparm_scal;
class spatial_parm;
class locn_info;

extern int prior;
extern locn_info* locations;
extern dataset* data;
extern int* idum;                              /* seed for random number generation during proposals */

const double highkappa_scale=1.5;
const double kappa_highval=0.75;


/* parameters are coded as having a 2-level hierarchy, with the higher level 
being hyperparm_scal and hyperparm_vec, the basic classes and the lower level 
being spatial_parm and nonspatial parm.  spatial_parm is a vector of parameters that are spatially correlated, with hyperparameters tying them together. */

/* variables named theta  are on the scale of the proposals and are usually normal proposal distributions; those named psi are the transformed values of theta used in the likelihood; those named ksi are matrix multiplications of theta that are not yet transformed and/or have not yet had a mean added; for hyperparm_scal, base_value is on scale of proposal and value is after transformation */


class nonspatial_parm{                      /* single scalar parameter */
public:
  nonspatial_parm(int which_in,double theta_init,double theta_mean, double theta_sd,double prop_sd_in,prior_dens_fxn_ptr prior_in,transform_fxn_ptr transform_in);
  ~nonspatial_parm(void);
  int step(void);                         /* one MCMC proposal */
  int accept(double logprior_diff);
  friend chain;                                 /* access to psi to pass to data (for delta) */
  /*  friend dataset; */
private:
  hyperparm_scal* psi;                        
  int which;                                    /* constant indicating which parameter it is */
  int num_accept;
  int run_num_accept;
};

class nonspatial_vecparm{                      /* single scalar parameter */
public:
  nonspatial_vecparm(int which_in,int num_in,double* theta_init,double* theta_mean, double* theta_sd,double prop_sd_in,prior_dens_fxn_ptr prior_in,transform_fxn_ptr transform_in);
  nonspatial_vecparm(int which_in,int num_in,double* theta_init,double theta_mean, double theta_sd,double prop_sd_in,prior_dens_fxn_ptr prior_in,transform_fxn_ptr transform_in);
  ~nonspatial_vecparm(void);
  int step(int mode);                         /* one MCMC proposal */
  int accept(double logprior_diff);
  void add_child(spatial_parm* alpha_in,spatial_parm* beta_in);
  double** ptrvec;
  friend chain;                                 /* access to psi to pass to data (for delta) */
  /*  friend dataset; */
private:
  hyperparm_scal** psi;                        
  int which;                                    /* constant indicating which parameter it is */
  int num;
  int num_accept;
  int run_num_accept;
  spatial_parm* alpha;      /* used by data cov parms for proposing alpha and beta with themselves */
  spatial_parm* beta;
};


class spatial_parm{                    /* spatially correlated vector of parameters */
public:
  spatial_parm(int which_in,int init_mode,double* init,double mu_init, double logsigma_init, double logkappa_init, double mu_mean, double mu_sd, double logsigma_mean, double logsigma_sd, double logkappa_mean,double logkappa_sd,double theta_prop_sd,double theta_prop_sd_subgroup,int subgroup_size_in, double mu_prop_sd, double logsigma_prop_sd, double logkappa_prop_sd, double ratio_in,transform_fxn_ptr transform_in,double kappa_alone_propsd_in);
  ~spatial_parm(void);
  /* void propose(void); */
  void step(int basic_mode,int langevin,int theta_prop_mode,int fastparm_fixed);      /* one MCMC proposal */
  void ksi_to_psi(void);                      /* calculate g(theta) -- for NON_DISCRETE */
  void ksi_to_theta(void);
  void theta_to_ksi(void);
  double ksi_propose(int update_chi);
  void add_child(spatial_parm* alpha_in,spatial_parm* beta_in);
  void add_info(void);

  int accept_nolik(double logprior_diff);               /* determines whether proposa  */
  int accept_lik(double logprior_diff);               /* determines whether proposal is accepted */
  int accept_lik(double logprior_diff,int print);               /* determines whether proposal is accepted */
  void postmean_update(int accepted);
  friend chain;                                 /* needs access to psi to pass value to data */
  friend nonspatial_vecparm;                   /* needed for access to alpha and beta during PM sampling */
  /* friend dataset; */
private:
  int which;
  int subgroup_size;
  /* constant indicating which parameter it is */
  int num_thetapoints;                          /* num_gridpoints if using DISCRETE method, num_datapoints otherwise */
  double ratio;                                 /* ratio of sds for log(kappa)/log(sigma) for S,D proposals */
  hyperparm_vec* theta;                         /* transformed indep. w~N(0,1) prior values */
  hyperparm_vec* psi;                           /* actual parameters: g(mu+Kw) */
  hyperparm_vec* ksi;                           /* Kw */
  hyperparm_vec* chi;

  sparse_cov_mat_stat* stat_cov;

  hyperparm_scal* sigma;                        /* log(sd) of theta */
  hyperparm_scal* mu;                           /* mean of psi (before transformation) */
  hyperparm_scal* kappa;                        /* sd of kernels */
  transform_fxn_ptr transform;                  /* g transformation function */

  spatial_parm* alpha;      /* used by data cov parms for proposing alpha and beta with themselves */
  spatial_parm* beta;

  int oldgrad_exceeded;
  int newgrad_exceeded;

  int num_accept_kappa;
  int num_accept_theta;
  int num_accept_theta_subgroup;
  int run_num_accept_kappa;
  int run_num_accept_theta;
  int run_num_accept_theta_subgroup;
  int num_accept_mu;
  int num_accept_sigma;
  int run_num_accept_mu;
  int run_num_accept_sigma;

  /* when doing multiple runs from dispersed starting points, need to propose kappa on its own to speed convergence; these variables deal with that */
  int num_accept_kappa2;
  int run_num_accept_kappa2;
  double kappa_alone_propsd;

  int num_dim;
  double* tmpvec;
  double* tmpvec2;
  double* oldgrad;
  double* newgrad;
  double* LCsum;
  double* Csum_diff;
  double logprior_diff;

};

class hyperparm_vec{                             /* vector hyperparameter - components are independent */
public:
  hyperparm_vec(int size_in,double* theta_init,double prop_sd_in); /*,double* sd_in);*/ /* use for theta */
  hyperparm_vec(int size_in,double* theta_init,double prop_sd_in,double prop_sd_subgroup_in); /*,double* sd_in);*/ /* use for theta */
  hyperparm_vec(int size_in,double prop_sd_in);       /* use for psi and ksi, i.e. for parameters that don't need to be proposed */
  hyperparm_vec(int size_in);       /* use for psi and ksi, i.e. for parameters that don't need to be proposed */
  ~hyperparm_vec(void);
  void update(int accepted);                     /* update current and stored parameter values based on whether proposal accepted */
  void propose(double alternate_prop_sd);                             /* generate proposal value */
  void propose(double alternate_prop_sd,int subgroup_size);                             /* generate proposal value */
  double evaluate_prior(void);                     /* calc prior density of proposal value */
  double* value;                                 /* current/proposed value */
  int subgroup_fixed;
  friend chain;                                  /* needs access to logprior */
  friend spatial_parm;                           /* needs access to prop_sd */
  int fixed;
  /*friend nonspatial_vecparm;*/                     /* needs to be able to update alpha and beta param vectors when doing PM sampling */
private:
  int size;                                      /* number of locations */
  transform_fxn_ptr transform;                  /* g transformation function */
  double prop_sd;                                /* proposal variance */
  double prop_sd_subgroup;
  double logprior;                               /* current prior density and density of proposed value */
  double logprior_stored;                        /* density of most recently accepted value */
  /*  double* sd;  */                                   /* prior sd of values - needed because it's used in proposal sd */
  double* value_stored;                          /* most recently accepted value */
};

class hyperparm_scal{                            /* scalar hyperparameter -- see comments above */
public:
  hyperparm_scal(double init,double parm1_in,double parm2_in,double prop_sd_in,prior_dens_fxn_ptr prior_dens_fxn_in,transform_fxn_ptr transform_in);
  hyperparm_scal(void);
  void update(int accepted);
  double evaluate_prior(void);
  double propose(double alternate_prop_sd);
  void propose(double alternate_prop_sd,double term);
  double value; 
  friend nonspatial_parm;
  friend spatial_parm;                          /* access to value_stored for spatial prior calc */
  friend chain;                                 /* needs access to value_stored to pass to dataset and access to logprior */
  friend nonspatial_vecparm;                    /* needs access to base_value for eigenvec to reset to (0,pi) */
  int fixed;                                    /* if fixed, parm is not sampled */
private:
  prior_dens_fxn_ptr prior_dens_fxn;
  transform_fxn_ptr transform;                  /* g transformation function */
  double prop_sd;
  double logprior;
  double logprior_stored;
  double base_value; 
  double base_value_stored; 
  double value_stored; 
  double parm1;                                      /* prior mean or lower limit */
  double parm2;                                  /* prior variance or upper limit */

};













