/* File:      transform.C
   Date:      12/31/01  (written fall 2001)
   Author:    C. Paciorek
   Purpose:   basic transformation functions - some taken from Num Recipes in C
   Compile:   g++ -c
   Usage:     
   Arguments: 
   Requires:  base.h
   Reference: Numerical Recipes in C
*/

#include "base.h"
//#include "qchisq.h"
#include "imsls.h"     // for inv_chi_squ fxn

// erfcc is taken from NR in C
double erf(double x) // Returns the error function erf(x) with fractional error everywhere less than 1:2   10  7 .   I've taken the complement of erfcc from NR in C
    { 
      double t,z,ans; 
      z=fabs(x); 
      t=1.0/(1.0+0.5*z); 
      ans=t*exp(-z*z-1.26551223+t*(1.00002368+t*(0.37409196+t*(0.09678418+ t*(-0.18628806+t*(0.27886807+t*(-1.13520398+t*(1.48851587+ t*(-0.82215223+t*0.17087277))))))))); 
      return x >= 0.0 ? 2.0 -ans : ans; 
    }

double erf(double x,double parm) // Returns the complementary error function erfc(x) with fractional error everywhere less than 1:2   10  7 . 
    { 
      double t,z,ans; 
      z=fabs(x); 
      t=1.0/(1.0+0.5*z); 
      ans=t*exp(-z*z-1.26551223+t*(1.00002368+t*(0.37409196+t*(0.09678418+ t*(-0.18628806+t*(0.27886807+t*(-1.13520398+t*(1.48851587+ t*(-0.82215223+t*0.17087277))))))))); 
      return x >= 0.0 ? 2.0 -ans : ans; 
    }

double identity(double in){
  return(in);
}

double identity(double in,double parm){
  return(in);
}

double log_transform(double in){
  return(exp(in));
}

double log_transform(double in,double parm){
  return(exp(in));
}

// Normal CDF multiplied by Pi to give value in (0,2*Pi)
double phi_transform_rad(double in){
  return(erf(in/SQRT2)*PI);  // phi(x) * 2 pi
}

double phi_transform_rad(double in,double parm){
  return(erf(in/SQRT2)*PI);  // phi(x) * 2 pi
}

// Normal CDF - gives value in (0,1)
double phi_transform(double in){
  return(erf(in/SQRT2)/2);
}

double phi_transform(double in,double parm){
  return(erf(in/SQRT2)/2);
}

// first calculates Normal CDF, then takes the quantile and calculates the point in a chi sq dist with parm df (sqrt of it)
double chisq_transform(double in,double parm){
  double unif=phi_transform(in,parm);
  return(sqrt(imsls_d_chi_squared_inverse_cdf(unif,parm)));
  //return(sqrt(qchisq(unif,parm)));
}


